################################################################################
#                                                                              # 
#                   Chemically Self-Propelled Janus Swimmers                   #
#                                                                              #
#         a Tcl/ESPResSo Script addepted from F. Luigli by J. de Graaf         #
#                                                                              # 
################################################################################

puts "##########################################"
puts "#                                        #" 
puts "#       Chemically Self-Propelled        #"
puts "#             Janus Swimmers             #"
puts "#                                        #" 
puts "##########################################\n"

################################################################################
#                               Output Directory                               #
################################################################################

puts "***     Creating output directory      ***\n"

set dirini "./DATA/"
set dirnex "run_ERROR"
set dir "$dirini$dirnex/"
file mkdir $dir

################################################################################
#                                  Constants                                   #
################################################################################

# Constants

set PI 3.14159265358979323844

# Set the precision with which 
# the Tcl values are output

set tcl_precision 10

################################################################################
#                                      Box                                     #
################################################################################

puts "***     Setting System Properties      ***\n"

# Box size

set a_boxl 50.0
set b_boxl 50.0
set c_boxl 50.0

setmd box_l $a_boxl $b_boxl $c_boxl

# Box volume

set box_vol [expr $a_boxl*$b_boxl*$c_boxl]

# Set the cell system

cellsystem domain_decomposition

# Set periodicity

setmd periodic 1 1 1

# Set the skin size

setmd skin 0.3

################################################################################
#                                    Swimmer                                   #
################################################################################

# Swimmer (ellipsoid) semi-axes

set a_axis 3.0
set b_axis $a_axis
set c_axis $a_axis

# Initial position of the swimmer

set x [expr 0.5*$a_boxl]
set y [expr 0.5*$b_boxl]
set z [expr 0.5*$c_boxl]

# The tolerance for the surface
# coverage of the swimmer

set tol 0.75

# Swimmer properties 
# (mass density, volume, and mass)

set rho_coll 1.0
set ellipsoid_vol [expr (4./3.)*$PI*$a_axis*$b_axis*$c_axis ]
set mass [expr $rho_coll*$ellipsoid_vol ]

################################################################################
#                                MD Integration                                #
################################################################################

# The time step, and the number of 
# equilibration and sample steps

set tstep 0.005  
set eqsteps 5000
set sampsteps 250000

# Set the time step

setmd time_step $tstep

################################################################################
#                              Reaction Properties                             #
################################################################################

# Reactant+Product concentration

set conc 0.25

# The reaction rate R+C -> P+C
# (Reactant, Colloid, Product)

set rate 1.00

# The minimum distance to the colloid
# for the reaction to occur

set dist_react 0.5

# The offset radius and the strength
# of the repulsion between the products
# and the catalyst particles

set r_c 1.0
set a_rep 10.0

################################################################################
#                                Particle Types                                #
################################################################################

# Set Particle Type 

set typecounter 0

# Lennard-Jones particles have type 0

set LJ $typecounter
incr typecounter

# Catalyst particles have type 1

set CATA $typecounter
incr typecounter

# Inert particles have type 2

set INER $typecounter
incr typecounter

# Reactant particles have type 3

set REAC $typecounter
incr typecounter

# Reaction product particles have type 4

set PROD  $typecounter
incr typecounter

################################################################################
#                            Particle Concentration                            #
################################################################################

# Determine the free volume available to the solvent,
# and the number of reactant and product particles
# that follow from this

set excl_vol [expr ($box_vol - $ellipsoid_vol)] 
set n_react_part [expr int($conc*$excl_vol)]

################################################################################
#                        Set the Interaction Parameters                        #
################################################################################

# Set the WCA interaction parameters

set eps_LJ 1.0
set off_LJ 0.0
set shi_LJ 0.25
set sig_LJ [expr $tol*$r_c]
set cut_LJ [expr (1.122462048*$sig_LJ + $off_LJ)]

# Ensure that the shell of 
# the colloid is hard

inter $INER $REAC lennard-jones $eps_LJ $sig_LJ $cut_LJ $shi_LJ $off_LJ
inter $INER $PROD lennard-jones $eps_LJ $sig_LJ $cut_LJ $shi_LJ $off_LJ
inter $CATA $REAC lennard-jones $eps_LJ $sig_LJ $cut_LJ $shi_LJ $off_LJ
inter $CATA $PROD lennard-jones $eps_LJ $sig_LJ $cut_LJ $shi_LJ $off_LJ

# Set the soft-sphere interaction between
# the products and the catalyst

inter $CATA $PROD hat $a_rep $r_c

################################################################################
#                        Compensate for Raspberry Model                        #
################################################################################

# Set particle radii, compensating 
# for the WCA potential

set na_axis [expr $a_axis - 0.5*$sig_LJ]
set nb_axis [expr $b_axis - 0.5*$sig_LJ]
set nc_axis [expr $c_axis - 0.5*$sig_LJ]

# Set new reaction range, compensating
# for the WCA potential

set ndist_react [expr $dist_react + 0.5*$sig_LJ]
set nz_react [expr sqrt($ndist_react*$ndist_react - 0.25*$sig_LJ*$sig_LJ)]

################################################################################
#                  Subroutine for the Creation of the Swimmer                  #
################################################################################

puts "***          Creating Swimmer          ***\n"

source swimmer_create.tcl

# Since the swimmer may unusually shaped
# ESPResSo may not set the proper global
# cut-off value

setmd min_global_cut [expr 1.05*$c_axis]

# Invoke subroutine to create the swimmer

create_swimmer $x $y $z

################################################################################
#                        Set up Lattice-Boltzmann Fluid                        #
################################################################################

puts "***   Initializing Lattice-Boltzmann   ***\n"

# Grid size (box lengths must be a 
# multiple of this value!)

#set grid_size 1.0

# Fluid density (~1.0)

#set density 1.0

# Fluid viscosity (0.5 - 5)

#set viscosity 3.0

# Friction (1-10)

set friction 10.0

# Initialize LB fluid

#lbfluid agrid $grid_size dens $density visc $viscosity \
#        tau $tstep friction $friction

################################################################################
#                              Set up Thermostat                               #
################################################################################

puts "***     Setting up the Thermostat      ***\n"

# The temperature of the system

set temperature 1.0

# Set the DPD Thermostat in ESPResSo

thermostat langevin $temperature $friction

################################################################################
#                   Place the Reactant and Product in the Box                  #
################################################################################

puts "***  Placing the Reactant and Product  ***\n"

for {set i 0} {$i < $n_react_part} {incr i} {

  # Start numbering from the last particle added

  set REACid [setmd n_part]

  # Catch the id number of the first and last
  # reactant particle

  if { $i==0 } { 
    set firstREACid $REACid } 
  if { $i==[expr $n_react_part-1] } { 
    set lastREACid $REACid }

  # While the particle is placed inside the 
  # ellipsoid, determine a new random position

  set dist 0.0
  while { $dist <=1.5 } {

    # Get a random position

    set posx [expr $a_boxl*[t_random]]
    set posy [expr $b_boxl*[t_random]]
    set posz [expr $c_boxl*[t_random]]

    # Determine the distance to the 
    # ellipsoid's centre of mass

    set dx [expr ($posx - $x)]
    set dy [expr ($posy - $y)]
    set dz [expr ($posz - $z)]
    set dist [expr (($dx*$dx)/($a_axis*$a_axis) \
                   +($dy*$dy)/($b_axis*$b_axis) \
                   +($dz*$dz)/($c_axis*$c_axis)) ]
  }

  # Set a new reactant/product particle at the 
  # position we just determined, with a random 
  # velocity, every second particle is a 
  # product (to help with equilibration)
 
  if { int($i % 2)==0 } {
    part $REACid pos $posx $posy $posz \
         type $REAC v [expr [t_random] - 0.5] \
         [expr [t_random] - 0.5] [expr [t_random] - 0.5]
  } else {
    part $REACid pos $posx $posy $posz \
         type $PROD v [expr [t_random] - 0.5] \
         [expr [t_random] - 0.5] [expr [t_random] - 0.5] 
  }
}

################################################################################
#                             Set up the Reaction                              #
################################################################################

##reaction reactant_type $REAC catalyzer_type $CATA \
##         product_type $PROD range $ndist_react rate $rate react_once on

################################################################################
#                          Equilibration of the System                         #
################################################################################

puts "***    Equilibration of the System     ***\n"

# Set the file for the equilibration data

set outdat "equilibration.dat"
set pro [open "$dir$outdat" "w"]

# Give the initial state of the system

set n_total [setmd n_part]
set deg_free [degrees_of_freedom]

puts $pro "# Total number of particles: $n_total"
puts $pro "# Number of reactant/product particles: $n_react_part"
puts $pro "# Number of swimmer particles: [expr $n_total - $n_react_part]"
puts $pro "# Thermostat: [thermostat]"
puts $pro "# i\ttime Ekin, temperature, distance, nr. of products"
puts $pro "#  \tswimmer position, velocity, force,"
puts $pro "#  \torientation, angular momentum, and torque"

# Equilibrate the system

for { set i 0 } { $i < $eqsteps } { incr i } {

  # Integrate one time step

  integrate 1

  # Every 100 steps we want to determine
  # the properties of the system 

  if { int($i % 10)==0 } {

    # Determine the temperature, time
    # and the minimum interparticle
    # distance 

    set en_kin [analyze energy kinetic]
    set temp [expr 2.0*$en_kin/($deg_free*(1.0 + 0.5*$n_react_part))]
    set t [expr $i*[setmd time_step]]

    # Determine how many of the particles 
    # have undergone a reaction

    set count 0
    for {set j $firstREACid} {$j < $lastREACid+1} {incr j} {
      set type [part $j print type]
      if {$type == $PROD} {incr count}
    }

    # Output our quantities

    puts $pro "$i $t $en_kin $temp $count \
               [part $LJid print pos v f quat omega torque]"
  }
}

# Close file streams

close $pro

################################################################################
#                                Production Run                                #
################################################################################

for {set k 0} {$k < 50} {incr k} {

  # Set the file for the production data

  set outdat "production"
  set outexi ".dat"
  set pro [open "$dir$outdat\_$k$outexi" "w"]
  puts $pro "# i\ttime, Ekin, temperature, nr. of products"
  puts $pro "#  \tswimmer position, velocity, force,"
  puts $pro "#  \torientation, angular momentum, and torque"

  # Set up the MSD calculation

  set pos_id [observable new particle_positions id $LJid]

  set tmax_1 [expr 500.0*$tstep]
  set msd_1 [correlation new obs $pos_id corr_operation square_distance_componentwise dt [expr 1.000*$tstep] tau_max $tmax_1 tau_lin 500]
  correlation $msd_1 autoupdate start

  set tmax_2 [expr 5000.0*$tstep]
  set msd_2 [correlation new obs $pos_id corr_operation square_distance_componentwise dt [expr 10.00*$tstep] tau_max $tmax_2 tau_lin 500]
  correlation $msd_2 autoupdate start

  set tmax_3 [expr 50000.0*$tstep]
  set msd_3 [correlation new obs $pos_id corr_operation square_distance_componentwise dt [expr 100.0*$tstep] tau_max $tmax_3 tau_lin 500]
  correlation $msd_3 autoupdate start

  # Set up the VACF calculation

  set vel_id [observable new particle_velocities id $LJid]

  set vacf_1 [correlation new obs $vel_id corr_operation scalar_product dt [expr 1.000*$tstep] tau_max $tmax_1 tau_lin 500]
  correlation $vacf_1 autoupdate start

  set vacf_2 [correlation new obs $vel_id corr_operation scalar_product dt [expr 10.00*$tstep] tau_max $tmax_2 tau_lin 500]
  correlation $vacf_2 autoupdate start

  set vacf_3 [correlation new obs $vel_id corr_operation scalar_product dt [expr 100.0*$tstep] tau_max $tmax_3 tau_lin 500]
  correlation $vacf_3 autoupdate start

  puts "***           Production Run           ***\n"

  for { set i 0 } { $i < $sampsteps } { incr i } {

    # Integrate one time step

    integrate 1

    # Every 100 steps we want to determine
    # the properties of the system 

    if { int($i % 100)==0 } {

      # Determine the temperature and time

      set en_kin [analyze energy kinetic]
      set temp [expr 2.0*$en_kin/($deg_free*(1.0 + 0.5*$n_react_part))]
      set t [expr $i*[setmd time_step]]

      # Determine how many of the particles 
      # have undergone a reaction

      set count 0
      for {set j $firstREACid} {$j < $lastREACid+1} {incr j} {
        set type [part $j print type]
        if {$type == $PROD} {incr count}
      }

      # Output our quantities

      puts $pro "$i $t $en_kin $temp $count \
                 [part $LJid print pos v f quat omega torque]"
    }
  }

  # Finalize the MSD and export

  correlation $msd_1 autoupdate stop
  correlation $msd_1 finalize
  set msdstr "msd_1"
  set msdexi ".dat"
  correlation $msd_1 write_to_file "$dir$msdstr\_$k$msdexi"

  correlation $msd_2 autoupdate stop
  correlation $msd_2 finalize
  set msdstr "msd_2"
  set msdexi ".dat"
  correlation $msd_2 write_to_file "$dir$msdstr\_$k$msdexi"

  correlation $msd_3 autoupdate stop
  correlation $msd_3 finalize
  set msdstr "msd_3"
  set msdexi ".dat"
  correlation $msd_3 write_to_file "$dir$msdstr\_$k$msdexi"

  # Finalize the VACF and export

  correlation $vacf_1 autoupdate stop
  correlation $vacf_1 finalize
  set vacfstr "vacf_1"
  set vacfexi ".dat"
  correlation $vacf_1 write_to_file "$dir$vacfstr\_$k$vacfexi"

  correlation $vacf_2 autoupdate stop
  correlation $vacf_2 finalize
  set vacfstr "vacf_2"
  set vacfexi ".dat"
  correlation $vacf_2 write_to_file "$dir$vacfstr\_$k$vacfexi"

  correlation $vacf_3 autoupdate stop
  correlation $vacf_3 finalize
  set vacfstr "vacf_3"
  set vacfexi ".dat"
  correlation $vacf_3 write_to_file "$dir$vacfstr\_$k$vacfexi"

  # Close file streams

  close $pro
}

# exit ESPResSo

puts "##########################################\n"
exit
